@@ -693,9 +693,9 @@ do {								\
 
 ]b4_c_function_def([yy_reduce_print], [static void],
 		   [[YYSTYPE *yyvsp], [yyvsp]],
-    b4_locations_if([[[YYLTYPE *yylsp], [yylsp]],])
-		   [[int yyrule], [yyrule]]m4_ifset([b4_parse_param], [,])
-		   b4_parse_param)[
+    b4_locations_if([[[YYLTYPE *yylsp], [yylsp]],
+		   ])[[int yyrule], [yyrule]]m4_ifset([b4_parse_param], [,
+		   ])b4_parse_param)[
 {
   int yynrhs = yyr2[yyrule];
   int yyi;
@@ -1480,7 +1480,8 @@ yyreturn:
   if (yymsg != yymsgbuf)
     YYSTACK_FREE (yymsg);
 #endif
-  return yyresult;
+  /* Make sure YYID is used.  */
+  return YYID (yyresult);
 ]}
 
 
