@@ -1,5 +1,5 @@
 /* Functions to support abitsets.
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
    Contributed by Michael Hayes (m.hayes@elec.canterbury.ac.nz).
 
 This program is free software; you can redistribute it and/or modify
@@ -14,15 +14,15 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifndef _ABITSET_H
-#define _ABITSET_H 
+#define _ABITSET_H
 
 #include "bitset.h"
 
-extern size_t abitset_bytes PARAMS ((bitset_bindex));
+extern size_t abitset_bytes (bitset_bindex);
 
-extern bitset abitset_init PARAMS ((bitset, bitset_bindex));
+extern bitset abitset_init (bitset, bitset_bindex);
 
 #endif
