@@ -1,5 +1,5 @@
 /* Associativity information.
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2005, 2006 Free Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
@@ -18,6 +18,7 @@
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.  */
 
+#include <config.h>
 #include "system.h"
 
 #include "assoc.h"
@@ -28,6 +29,9 @@ assoc_to_string (assoc a)
 {
   switch (a)
     {
+    default:
+      abort ();
+
     case undef_assoc:
       return "undefined associativity";
 
@@ -40,5 +44,4 @@ assoc_to_string (assoc a)
     case non_assoc:
       return "%nonassoc";
     }
-  return NULL;
 }
