@@ -1,5 +1,5 @@
 /* Parse command line arguments for bison.
-   Copyright (C) 1984, 1986, 1989, 1992, 2000, 2001, 2002
+   Copyright (C) 1984, 1986, 1989, 1992, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
@@ -16,24 +16,48 @@
 
    You should have received a copy of the GNU General Public License
    along with Bison; see the file COPYING.  If not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 #ifndef GETARGS_H_
 # define GETARGS_H_
 
 /* flags set by % directives */
-extern const char *skeleton;		/* for -S */
-extern const char *include;		/* for -I */
 
-extern int debug_flag; 		/* for -t */
-extern int defines_flag;    	/* for -d */
-extern int locations_flag;
-extern int no_lines_flag;    	/* for -l */
-extern int no_parser_flag;	/* for -n */
-extern int token_table_flag;   	/* for -k */
-extern int graph_flag;		/* for -g */
-extern int yacc_flag;  		/* for -y */
+/* for -S */
+extern char const *skeleton;
+
+/* for -I */
+extern char const *include;
+
+extern bool debug_flag;			/* for -t */
+extern bool defines_flag;		/* for -d */
+extern bool graph_flag;			/* for -g */
+extern bool locations_flag;
+extern bool no_lines_flag;		/* for -l */
+extern bool no_parser_flag;		/* for -n */
+extern bool token_table_flag;		/* for -k */
+extern bool yacc_flag;			/* for -y */
+
+extern bool error_verbose;
+
+
+/* GLR_PARSER is true if the input file says to use the GLR
+   (Generalized LR) parser, and to output some additional information
+   used by the GLR algorithm.  */
+
+extern bool glr_parser;
+
+/* PURE_PARSER is true if should generate a parser that is all pure
+   and reentrant.  */
+
+extern bool pure_parser;
+
+/* NONDETERMINISTIC_PARSER is true iff conflicts are accepted.  This
+   is used by the GLR parser, and might be used in BackTracking
+   parsers too.  */
+
+extern bool nondeterministic_parser;
 
 /* --trace.  */
 enum trace
@@ -49,6 +73,7 @@ enum trace
     trace_grammar   = 1 << 7,
     trace_time      = 1 << 8,
     trace_skeleton  = 1 << 9,
+    trace_m4        = 1 << 10,
     trace_all       = ~0
   };
 extern int trace_flag;
@@ -59,7 +84,7 @@ enum report
     report_none             = 0,
     report_states           = 1 << 0,
     report_itemsets         = 1 << 1,
-    report_lookaheads       = 1 << 2,
+    report_look_ahead_tokens= 1 << 2,
     report_solved_conflicts = 1 << 3,
     report_all              = ~0
   };
