@@ -1,5 +1,7 @@
-/* Symbol table manager for Bison,
-   Copyright (C) 1984, 1989, 2000, 2001, 2002 Free Software Foundation, Inc.
+/* Symbol table manager for Bison.
+
+   Copyright (C) 1984, 1989, 2000, 2001, 2002, 2004, 2005, 2006 Free
+   Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
@@ -15,39 +17,42 @@
 
    You should have received a copy of the GNU General Public License
    along with Bison; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
-
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
+#include <config.h>
 #include "system.h"
-#include "quotearg.h"
-#include "hash.h"
+
+#include <hash.h>
+#include <quotearg.h>
+
 #include "complain.h"
-#include "symtab.h"
 #include "gram.h"
+#include "symtab.h"
 
 /*------------------------.
 | Distinguished symbols.  |
 `------------------------*/
 
-symbol_t *errtoken = NULL;
-symbol_t *undeftoken = NULL;
-symbol_t *endtoken = NULL;
-symbol_t *accept = NULL;
-symbol_t *startsymbol = NULL;
-location_t startsymbol_location;
+symbol *errtoken = NULL;
+symbol *undeftoken = NULL;
+symbol *endtoken = NULL;
+symbol *accept = NULL;
+symbol *startsymbol = NULL;
+location startsymbol_location;
 
 /*---------------------------------.
 | Create a new symbol, named TAG.  |
 `---------------------------------*/
 
-static symbol_t *
-symbol_new (const char *tag, location_t location)
+static symbol *
+symbol_new (uniqstr tag, location loc)
 {
-  symbol_t *res = XMALLOC (symbol_t, 1);
+  symbol *res = xmalloc (sizeof *res);
 
-  res->tag = xstrdup (tag);
-  res->location = location;
+  uniqstr_assert (tag);
+  res->tag = tag;
+  res->location = loc;
 
   res->type_name = NULL;
   res->destructor = NULL;
@@ -60,131 +65,175 @@ symbol_new (const char *tag, location_t location)
 
   res->alias = NULL;
   res->class = unknown_sym;
+  res->declared = false;
 
+  if (nsyms == SYMBOL_NUMBER_MAXIMUM)
+    fatal (_("too many symbols in input grammar (limit is %d)"),
+	   SYMBOL_NUMBER_MAXIMUM);
   nsyms++;
   return res;
 }
 
 
-/*------------------------------------------------------------------.
-| Set the TYPE_NAME associated to SYMBOL.  Does nothing if passed 0 |
-| as TYPE_NAME.                                                     |
-`------------------------------------------------------------------*/
+/*-----------------.
+| Print a symbol.  |
+`-----------------*/
+
+#define SYMBOL_ATTR_PRINT(Attr)				\
+  if (s->Attr)						\
+    fprintf (f, " %s { %s }", #Attr, s->Attr)
 
 void
-symbol_type_set (symbol_t *symbol, char *type_name, location_t location)
+symbol_print (symbol *s, FILE *f)
+{
+  if (s)
+    {
+      fprintf (f, "\"%s\"", s->tag);
+      SYMBOL_ATTR_PRINT (type_name);
+      SYMBOL_ATTR_PRINT (destructor);
+      SYMBOL_ATTR_PRINT (printer);
+    }
+  else
+    fprintf (f, "<NULL>");
+}
+
+#undef SYMBOL_ATTR_PRINT
+
+/*------------------------------------------------------------------.
+| Complain that S's WHAT is redeclared at SECOND, and was first set |
+| at FIRST.                                                         |
+`------------------------------------------------------------------*/
+
+static void
+redeclaration (symbol* s, const char *what, location first, location second)
+{
+  complain_at (second, _("%s redeclaration for %s"), what, s->tag);
+  complain_at (first, _("first declaration"));
+}
+
+
+/*-----------------------------------------------------------------.
+| Set the TYPE_NAME associated with SYM.  Does nothing if passed 0 |
+| as TYPE_NAME.                                                    |
+`-----------------------------------------------------------------*/
+
+void
+symbol_type_set (symbol *sym, uniqstr type_name, location loc)
 {
   if (type_name)
     {
-      if (symbol->type_name)
-	complain_at (location,
-		     _("type redeclaration for %s"), symbol->tag);
-      symbol->type_name = type_name;
-    }
-}
-
-
-/*-------------------------------------------------------------------.
-| Set the DESTRUCTOR associated to SYMBOL.  Do nothing if passed 0.  |
-`-------------------------------------------------------------------*/
-
-void
-symbol_destructor_set (symbol_t *symbol, char *destructor, location_t location)
-{
-  if (destructor)
-    {
-      if (symbol->destructor)
-	complain_at (location,
-		     _("%s redeclaration for %s"),
-		     "%destructor", symbol->tag);
-      symbol->destructor = destructor;
-      symbol->destructor_location = location;
-    }
-}
-
-
-/*----------------------------------------------------------------.
-| Set the PRITNER associated to SYMBOL.  Do nothing if passed 0.  |
-`----------------------------------------------------------------*/
-
-void
-symbol_printer_set (symbol_t *symbol, char *printer, location_t location)
-{
-  if (printer)
-    {
-      if (symbol->printer)
-	complain_at (location,
-		     _("%s redeclaration for %s"),
-		     "%printer", symbol->tag);
-      symbol->printer = printer;
-      symbol->printer_location = location;
+      if (sym->type_name)
+	redeclaration (sym, "%type", sym->type_location, loc);
+      uniqstr_assert (type_name);
+      sym->type_name = type_name;
+      sym->type_location = loc;
     }
 }
 
 
 /*------------------------------------------------------------------.
-| Set the PRECEDENCE associated to SYMBOL.  Does nothing if invoked |
-| with UNDEF_ASSOC as ASSOC.                                        |
+| Set the DESTRUCTOR associated with SYM.  Do nothing if passed 0.  |
 `------------------------------------------------------------------*/
 
 void
-symbol_precedence_set (symbol_t *symbol,
-		       int prec, assoc_t assoc, location_t location)
+symbol_destructor_set (symbol *sym, const char *destructor, location loc)
 {
-  if (assoc != undef_assoc)
+  if (destructor)
     {
-      if (symbol->prec != 0)
-	complain_at (location,
-		     _("redefining precedence of %s"),
-		     symbol->tag);
-      symbol->prec = prec;
-      symbol->assoc = assoc;
+      if (sym->destructor)
+	redeclaration (sym, "%destructor", sym->destructor_location, loc);
+      sym->destructor = destructor;
+      sym->destructor_location = loc;
+    }
+}
+
+
+/*---------------------------------------------------------------.
+| Set the PRINTER associated with SYM.  Do nothing if passed 0.  |
+`---------------------------------------------------------------*/
+
+void
+symbol_printer_set (symbol *sym, const char *printer, location loc)
+{
+  if (printer)
+    {
+      if (sym->printer)
+	redeclaration (sym, "%printer", sym->destructor_location, loc);
+      sym->printer = printer;
+      sym->printer_location = loc;
+    }
+}
+
+
+/*-----------------------------------------------------------------.
+| Set the PRECEDENCE associated with SYM.  Does nothing if invoked |
+| with UNDEF_ASSOC as ASSOC.                                       |
+`-----------------------------------------------------------------*/
+
+void
+symbol_precedence_set (symbol *sym, int prec, assoc a, location loc)
+{
+  if (a != undef_assoc)
+    {
+      if (sym->prec != 0)
+	redeclaration (sym, assoc_to_string (a), sym->prec_location, loc);
+      sym->prec = prec;
+      sym->assoc = a;
+      sym->prec_location = loc;
     }
 
   /* Only terminals have a precedence. */
-  symbol_class_set (symbol, token_sym, location);
+  symbol_class_set (sym, token_sym, loc, false);
 }
 
 
-/*-------------------------------------.
-| Set the CLASS associated to SYMBOL.  |
-`-------------------------------------*/
+/*------------------------------------.
+| Set the CLASS associated with SYM.  |
+`------------------------------------*/
 
 void
-symbol_class_set (symbol_t *symbol, symbol_class class, location_t location)
+symbol_class_set (symbol *sym, symbol_class class, location loc, bool declaring)
 {
-  if (symbol->class != unknown_sym && symbol->class != class)
-    complain_at (location, _("symbol %s redefined"), symbol->tag);
+  if (sym->class != unknown_sym && sym->class != class)
+    {
+      complain_at (loc, _("symbol %s redefined"), sym->tag);
+      sym->declared = false;
+    }
 
-  if (class == nterm_sym && symbol->class != nterm_sym)
-    symbol->number = nvars++;
-  else if (class == token_sym && symbol->number == NUMBER_UNDEFINED)
-    symbol->number = ntokens++;
+  if (class == nterm_sym && sym->class != nterm_sym)
+    sym->number = nvars++;
+  else if (class == token_sym && sym->number == NUMBER_UNDEFINED)
+    sym->number = ntokens++;
 
-  symbol->class = class;
+  sym->class = class;
+
+  if (declaring)
+    {
+      if (sym->declared)
+	warn_at (loc, _("symbol %s redeclared"), sym->tag);
+      sym->declared = true;
+    }
 }
 
 
-/*-------------------------------------------------.
-| Set the USER_TOKEN_NUMBER associated to SYMBOL.  |
-`-------------------------------------------------*/
+/*------------------------------------------------.
+| Set the USER_TOKEN_NUMBER associated with SYM.  |
+`------------------------------------------------*/
 
 void
-symbol_user_token_number_set (symbol_t *symbol,
-			      int user_token_number, location_t location)
+symbol_user_token_number_set (symbol *sym, int user_token_number, location loc)
 {
-  assert (symbol->class == token_sym);
+  assert (sym->class == token_sym);
 
-  if (symbol->user_token_number != USER_NUMBER_UNDEFINED
-      && symbol->user_token_number != user_token_number)
-    complain_at (location, _("redefining user token number of %s"),
-		 symbol->tag);
+  if (sym->user_token_number != USER_NUMBER_UNDEFINED
+      && sym->user_token_number != user_token_number)
+    complain_at (loc, _("redefining user token number of %s"), sym->tag);
 
-  symbol->user_token_number = user_token_number;
+  sym->user_token_number = user_token_number;
   /* User defined $end token? */
   if (user_token_number == 0)
     {
-      endtoken = symbol;
+      endtoken = sym;
       endtoken->number = 0;
       /* It is always mapped to 0, so it was already counted in
 	 NTOKENS.  */
@@ -193,72 +242,62 @@ symbol_user_token_number_set (symbol_t *symbol,
 }
 
 
-/*------------.
-| Free THIS.  |
-`------------*/
+/*----------------------------------------------------------.
+| If SYM is not defined, report an error, and consider it a |
+| nonterminal.                                              |
+`----------------------------------------------------------*/
 
-static void
-symbol_free (symbol_t *this)
+static inline bool
+symbol_check_defined (symbol *sym)
 {
-#if 0
-  /* This causes crashes because one string can appear more
-     than once.  */
-  XFREE (this->type_name);
-#endif
-  XFREE (this->tag);
-  XFREE (this);
-}
-
-
-/*-----------------------------------------------------------.
-| If THIS is not defined, report an error, and consider it a |
-| nonterminal.                                               |
-`-----------------------------------------------------------*/
-
-static bool
-symbol_check_defined (symbol_t *this)
-{
-  if (this->class == unknown_sym)
+  if (sym->class == unknown_sym)
     {
       complain_at
-	(this->location,
+	(sym->location,
 	 _("symbol %s is used, but is not defined as a token and has no rules"),
-	 this->tag);
-      this->class = nterm_sym;
-      this->number = nvars++;
+	 sym->tag);
+      sym->class = nterm_sym;
+      sym->number = nvars++;
     }
 
-  return TRUE;
+  return true;
+}
+
+static bool
+symbol_check_defined_processor (void *sym, void *null ATTRIBUTE_UNUSED)
+{
+  return symbol_check_defined (sym);
 }
 
 
-/*-------------------------------------------------------------------.
-| Declare the new SYMBOL.  Make it an alias of SYMVAL, and type them |
-| with TYPENAME.                                                     |
-`-------------------------------------------------------------------*/
+/*------------------------------------------------------------------.
+| Declare the new symbol SYM.  Make it an alias of SYMVAL, and type |
+| SYMVAL with SYM's type.                                           |
+`------------------------------------------------------------------*/
 
 void
-symbol_make_alias (symbol_t *symbol, symbol_t *symval, location_t loc)
+symbol_make_alias (symbol *sym, symbol *symval, location loc)
 {
   if (symval->alias)
     warn_at (loc, _("symbol `%s' used more than once as a literal string"),
-	  symval->tag);
-  else if (symbol->alias)
+	     symval->tag);
+  else if (sym->alias)
     warn_at (loc, _("symbol `%s' given more than one literal string"),
-	  symbol->tag);
+	     sym->tag);
   else
     {
       symval->class = token_sym;
-      symval->user_token_number = symbol->user_token_number;
-      symbol->user_token_number = USER_NUMBER_ALIAS;
-      symval->alias = symbol;
-      symbol->alias = symval;
-      /* symbol and symval combined are only one symbol */
+      symval->user_token_number = sym->user_token_number;
+      sym->user_token_number = USER_NUMBER_ALIAS;
+      symval->alias = sym;
+      sym->alias = symval;
+      /* sym and symval combined are only one symbol.  */
       nsyms--;
       ntokens--;
-      assert (ntokens == symbol->number || ntokens == symval->number);
-      symbol->number = symval->number =
-	(symval->number < symbol->number) ? symval->number : symbol->number;
+      assert (ntokens == sym->number || ntokens == symval->number);
+      sym->number = symval->number =
+	(symval->number < sym->number) ? symval->number : sym->number;
+      symbol_type_set (symval, sym->type_name, loc);
     }
 }
 
@@ -268,38 +307,60 @@ symbol_make_alias (symbol_t *symbol, symbol_t *symval, location_t loc)
 | associativity.                                           |
 `---------------------------------------------------------*/
 
-static bool
-symbol_check_alias_consistence (symbol_t *this)
+static inline void
+symbol_check_alias_consistency (symbol *this)
 {
-  /* Check only those who _are_ the aliases. */
-  if (this->alias && this->user_token_number == USER_NUMBER_ALIAS)
-    {
-      if (this->prec != this->alias->prec)
-	{
-	  if (this->prec != 0 && this->alias->prec != 0)
-	    complain_at (this->alias->location,
-			 _("conflicting precedences for %s and %s"),
-			 this->tag, this->alias->tag);
-	  if (this->prec != 0)
-	    this->alias->prec = this->prec;
-	  else
-	    this->prec = this->alias->prec;
-	}
+  symbol *alias = this;
+  symbol *orig  = this->alias;
 
-      if (this->assoc != this->alias->assoc)
-	{
-	  if (this->assoc != undef_assoc && this->alias->assoc != undef_assoc)
-	    complain_at (this->alias->location,
-			 _("conflicting associativities for %s (%s) and %s (%s)"),
-			 this->tag, assoc_to_string (this->assoc),
-			 this->alias->tag, assoc_to_string (this->alias->assoc));
-	  if (this->assoc != undef_assoc)
-	    this->alias->assoc = this->assoc;
-	  else
-	    this->assoc = this->alias->assoc;
-	}
+  /* Check only those that _are_ the aliases.  */
+  if (!(this->alias && this->user_token_number == USER_NUMBER_ALIAS))
+    return;
+
+  if (orig->type_name != alias->type_name)
+    {
+      if (orig->type_name)
+	symbol_type_set (alias, orig->type_name, orig->type_location);
+      else
+	symbol_type_set (orig, alias->type_name, alias->type_location);
     }
-  return TRUE;
+
+
+  if (orig->destructor || alias->destructor)
+    {
+      if (orig->destructor)
+	symbol_destructor_set (alias, orig->destructor,
+			       orig->destructor_location);
+      else
+	symbol_destructor_set (orig, alias->destructor,
+			       alias->destructor_location);
+    }
+
+  if (orig->printer || alias->printer)
+    {
+      if (orig->printer)
+	symbol_printer_set (alias, orig->printer, orig->printer_location);
+      else
+	symbol_printer_set (orig, alias->printer, alias->printer_location);
+    }
+
+  if (alias->prec || orig->prec)
+    {
+      if (orig->prec)
+	symbol_precedence_set (alias, orig->prec, orig->assoc,
+			       orig->prec_location);
+      else
+	symbol_precedence_set (orig, alias->prec, alias->assoc,
+			       alias->prec_location);
+    }
+}
+
+static bool
+symbol_check_alias_consistency_processor (void *this,
+					  void *null ATTRIBUTE_UNUSED)
+{
+  symbol_check_alias_consistency (this);
+  return true;
 }
 
 
@@ -308,8 +369,8 @@ symbol_check_alias_consistence (symbol_t *this)
 | into FDEFINES.  Put in SYMBOLS.                                    |
 `-------------------------------------------------------------------*/
 
-static bool
-symbol_pack (symbol_t *this)
+static inline bool
+symbol_pack (symbol *this)
 {
   if (this->class == nterm_sym)
     {
@@ -330,17 +391,21 @@ symbol_pack (symbol_t *this)
 	      this->number = this->alias->number;
 	    }
 	}
-      /* Do not do processing below for USER_NUMBER_ALIASs.  */
+      /* Do not do processing below for USER_NUMBER_ALIASes.  */
       if (this->user_token_number == USER_NUMBER_ALIAS)
-	return TRUE;
+	return true;
     }
   else /* this->class == token_sym */
-    {
-      assert (this->number != NUMBER_UNDEFINED);
-    }
+    assert (this->number != NUMBER_UNDEFINED);
 
   symbols[this->number] = this;
-  return TRUE;
+  return true;
+}
+
+static bool
+symbol_pack_processor (void *this, void *null ATTRIBUTE_UNUSED)
+{
+  return symbol_pack (this);
 }
 
 
@@ -350,8 +415,8 @@ symbol_pack (symbol_t *this)
 | Put THIS in TOKEN_TRANSLATIONS if it is a token.  |
 `--------------------------------------------------*/
 
-static bool
-symbol_translation (symbol_t *this)
+static inline bool
+symbol_translation (symbol *this)
 {
   /* Non-terminal? */
   if (this->class == token_sym
@@ -367,7 +432,13 @@ symbol_translation (symbol_t *this)
       token_translations[this->user_token_number] = this->number;
     }
 
-  return TRUE;
+  return true;
+}
+
+static bool
+symbol_translation_processor (void *this, void *null ATTRIBUTE_UNUSED)
+{
+  return symbol_translation (this);
 }
 
 
@@ -380,16 +451,30 @@ symbol_translation (symbol_t *this)
 
 static struct hash_table *symbol_table = NULL;
 
-static bool
-hash_compare_symbol_t (const symbol_t *m1, const symbol_t *m2)
+static inline bool
+hash_compare_symbol (const symbol *m1, const symbol *m2)
 {
-  return strcmp (m1->tag, m2->tag) ? FALSE : TRUE;
+  /* Since tags are unique, we can compare the pointers themselves.  */
+  return UNIQSTR_EQ (m1->tag, m2->tag);
 }
 
-static unsigned int
-hash_symbol_t (const symbol_t *m, unsigned int tablesize)
+static bool
+hash_symbol_comparator (void const *m1, void const *m2)
 {
-  return hash_string (m->tag, tablesize);
+  return hash_compare_symbol (m1, m2);
+}
+
+static inline size_t
+hash_symbol (const symbol *m, size_t tablesize)
+{
+  /* Since tags are unique, we can hash the pointer itself.  */
+  return ((uintptr_t) m->tag) % tablesize;
+}
+
+static size_t
+hash_symbol_hasher (void const *m, size_t tablesize)
+{
+  return hash_symbol (m, tablesize);
 }
 
 
@@ -402,9 +487,9 @@ symbols_new (void)
 {
   symbol_table = hash_initialize (HT_INITIAL_CAPACITY,
 				  NULL,
-				  (Hash_hasher) hash_symbol_t,
-				  (Hash_comparator) hash_compare_symbol_t,
-				  (Hash_data_freer) symbol_free);
+				  hash_symbol_hasher,
+				  hash_symbol_comparator,
+				  free);
 }
 
 
@@ -413,21 +498,20 @@ symbols_new (void)
 | yet, create it.                                                 |
 `----------------------------------------------------------------*/
 
-symbol_t *
-symbol_get (const char *key, location_t location)
+symbol *
+symbol_get (const char *key, location loc)
 {
-  symbol_t probe;
-  symbol_t *entry;
+  symbol probe;
+  symbol *entry;
 
-  /* Keep the symbol in a printable form.  */
-  key = quotearg_style (escape_quoting_style, key);
-  *(char const **) &probe.tag = key;
+  key = uniqstr_new (key);
+  probe.tag = key;
   entry = hash_lookup (symbol_table, &probe);
 
   if (!entry)
     {
       /* First insertion in the hash. */
-      entry = symbol_new (key, location);
+      entry = symbol_new (key, loc);
       hash_insert (symbol_table, entry);
     }
   return entry;
@@ -439,17 +523,17 @@ symbol_get (const char *key, location_t location)
 | user's names.                                                     |
 `------------------------------------------------------------------*/
 
-symbol_t *
-dummy_symbol_get (location_t location)
+symbol *
+dummy_symbol_get (location loc)
 {
   /* Incremented for each generated symbol.  */
   static int dummy_count = 0;
   static char buf[256];
 
-  symbol_t *sym;
+  symbol *sym;
 
   sprintf (buf, "@%d", ++dummy_count);
-  sym = symbol_get (buf, location);
+  sym = symbol_get (buf, loc);
   sym->class = nterm_sym;
   sym->number = nvars++;
   return sym;
@@ -473,12 +557,10 @@ symbols_free (void)
 | terminals.                                                     |
 `---------------------------------------------------------------*/
 
-void
-symbols_do (symbol_processor processor, void *processor_data)
+static void
+symbols_do (Hash_processor processor, void *processor_data)
 {
-  hash_do_for_each (symbol_table,
-		    (Hash_processor) processor,
-		    processor_data);
+  hash_do_for_each (symbol_table, processor, processor_data);
 }
 
 
@@ -490,7 +572,7 @@ symbols_do (symbol_processor processor, void *processor_data)
 void
 symbols_check_defined (void)
 {
-  symbols_do (symbol_check_defined, NULL);
+  symbols_do (symbol_check_defined_processor, NULL);
 }
 
 /*------------------------------------------------------------------.
@@ -501,7 +583,7 @@ symbols_check_defined (void)
 static void
 symbols_token_translations_init (void)
 {
-  int num_256_available_p = TRUE;
+  bool num_256_available_p = true;
   int i;
 
   /* Find the highest user token number, and whether 256, the POSIX
@@ -509,13 +591,13 @@ symbols_token_translations_init (void)
   max_user_token_number = 0;
   for (i = 0; i < ntokens; ++i)
     {
-      symbol_t *this = symbols[i];
+      symbol *this = symbols[i];
       if (this->user_token_number != USER_NUMBER_UNDEFINED)
 	{
 	  if (this->user_token_number > max_user_token_number)
 	    max_user_token_number = this->user_token_number;
 	  if (this->user_token_number == 256)
-	    num_256_available_p = FALSE;
+	    num_256_available_p = false;
 	}
     }
 
@@ -530,21 +612,22 @@ symbols_token_translations_init (void)
 
   for (i = 0; i < ntokens; ++i)
     {
-      symbol_t *this = symbols[i];
+      symbol *this = symbols[i];
       if (this->user_token_number == USER_NUMBER_UNDEFINED)
 	this->user_token_number = ++max_user_token_number;
       if (this->user_token_number > max_user_token_number)
 	max_user_token_number = this->user_token_number;
     }
 
-  token_translations = XCALLOC (symbol_number_t, max_user_token_number + 1);
+  token_translations = xnmalloc (max_user_token_number + 1,
+				 sizeof *token_translations);
 
   /* Initialize all entries for literal tokens to 2, the internal
      token number for $undefined, which represents all invalid inputs.
      */
   for (i = 0; i < max_user_token_number + 1; i++)
     token_translations[i] = undeftoken->number;
-  symbols_do (symbol_translation, NULL);
+  symbols_do (symbol_translation_processor, NULL);
 }
 
 
@@ -556,10 +639,10 @@ symbols_token_translations_init (void)
 void
 symbols_pack (void)
 {
-  symbols = XCALLOC (symbol_t *, nsyms);
+  symbols = xcalloc (nsyms, sizeof *symbols);
 
-  symbols_do (symbol_check_alias_consistence, NULL);
-  symbols_do (symbol_pack, NULL);
+  symbols_do (symbol_check_alias_consistency_processor, NULL);
+  symbols_do (symbol_pack_processor, NULL);
 
   symbols_token_translations_init ();
 
